@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,7 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
 import renpy.display
 import renpy.text
 import codecs
@@ -51,6 +52,8 @@ image_prefixes = None
 report_node = None
 
 # Reports a message to the user.
+
+
 def report(msg, *args):
     if report_node:
         out = u"%s:%d " % (renpy.parser.unicode_filename(report_node.filename), report_node.linenumber)
@@ -58,17 +61,19 @@ def report(msg, *args):
         out = ""
 
     out += msg % args
-    print
-    print out.encode('utf-8')
+    print()
+    print(out.encode('utf-8'))
 
 added = { }
 
 # Reports additional information about a message, the first time it
 # occurs.
+
+
 def add(msg):
     if not msg in added:
         added[msg] = True
-        print unicode(msg).encode('utf-8')
+        print(unicode(msg).encode('utf-8'))
 
 
 # Trys to evaluate an expression, announcing an error if it fails.
@@ -90,12 +95,14 @@ def try_eval(where, expr, additional=None):
     if m.group(1) in __builtins__:
         return
 
-    report( "Could not evaluate '%s', in %s.", expr, where)
+    report("Could not evaluate '%s', in %s.", expr, where)
     if additional:
         add(additional)
 
 # Returns True of the expression can be compiled as python, False
 # otherwise.
+
+
 def try_compile(where, expr, additional=None):
 
     try:
@@ -109,6 +116,7 @@ def try_compile(where, expr, additional=None):
 # The sets of names + attributes that we know are valid.
 imprecise_cache = set()
 
+
 def image_exists_imprecise(name):
     """
     Returns true if the image is a plausible image that can be used in a show
@@ -116,7 +124,6 @@ def image_exists_imprecise(name):
     tag and containing all of the attributes (and none of the removed attributes).
     """
 
-
     if name in imprecise_cache:
         return True
 
@@ -152,6 +159,7 @@ def image_exists_imprecise(name):
 
 precise_cache = set()
 
+
 def image_exists_precise(name):
     """
     Returns true if an image exists with the same tag and attributes as
@@ -217,6 +225,7 @@ def image_exists(name, expression, tag, precise=True):
 # Only check each file once.
 check_file_cache = { }
 
+
 def check_file(what, fn):
 
     present = check_file_cache.get(fn, None)
@@ -245,7 +254,7 @@ def check_displayable(what, d):
 
     try:
         if isinstance(d, renpy.display.core.Displayable):
-            d.visit_all(lambda a : a.predict_one())
+            d.visit_all(lambda a: a.predict_one())
     except:
         pass
 
@@ -260,6 +269,7 @@ def check_image(node):
 
     check_displayable('image %s' % name, renpy.display.image.images[node.imgname])
 
+
 def imspec(t):
     if len(t) == 3:
         return t[0], None, None, t[1], t[2], 0
@@ -278,13 +288,15 @@ def check_show(node, precise):
 
     name, expression, tag, at_list, layer, _zorder, _behind = imspec(node.imspec)
 
+    layer = renpy.exports.default_layer(layer, tag or name)
+
     if layer not in renpy.config.layers and layer not in renpy.config.top_layers:
         report("Uses layer '%s', which is not in config.layers.", layer)
 
     image_exists(name, expression, tag, precise=precise)
 
     for i in at_list:
-        try_eval("the at list of a scene or show statment", i, "Perhaps you forgot to declare, or misspelled, a position?")
+        try_eval("the at list of a scene or show statment", i, "Perhaps you forgot to define or misspelled a transform.")
 
 
 def precheck_show(node):
@@ -304,18 +316,19 @@ def check_hide(node):
 
     tag = tag or name[0]
 
+    layer = renpy.exports.default_layer(layer, tag)
+
     if layer not in renpy.config.layers and layer not in renpy.config.top_layers:
         report("Uses layer '%s', which is not in config.layers.", layer)
 
     if tag not in image_prefixes:
         report("The image tag '%s' is not the prefix of a declared image, nor was it used in a show statement before this hide statement.", tag)
 
-    # for i in at_list:
-    #    try_eval(node, "at list of hide statment", i)
 
 def check_with(node):
     try_eval("a with statement or clause", node.expr, "Perhaps you forgot to declare, or misspelled, a transition?")
 
+
 def check_user(node):
 
     def error(msg):
@@ -332,6 +345,7 @@ def check_user(node):
     except:
         report("Didn't properly report what the next statement should be.")
 
+
 def text_checks(s):
     msg = renpy.text.extras.check_text_tags(s)
     if msg:
@@ -374,6 +388,7 @@ def text_checks(s):
         if state != 0:
             report("Unterminated string format code '%s' (in %s)", fmt, repr(s)[1:])
 
+
 def check_say(node):
 
     if node.who:
@@ -430,6 +445,7 @@ def check_menu(node):
 
         text_checks(l)
 
+
 def check_jump(node):
 
     if node.expression:
@@ -438,6 +454,7 @@ def check_jump(node):
     if not renpy.game.script.has_label(node.target):
         report("The jump is to nonexistent label '%s'.", node.target)
 
+
 def check_call(node):
 
     if node.expression:
@@ -446,14 +463,17 @@ def check_call(node):
     if not renpy.game.script.has_label(node.label):
         report("The call is to nonexistent label '%s'.", node.label)
 
+
 def check_while(node):
     try_compile("in the condition of the while statement", node.condition)
 
+
 def check_if(node):
 
     for condition, _block in node.entries:
         try_compile("in a condition of the if statement", condition)
 
+
 def check_define(node, kind):
     if node.store != 'store':
         return
@@ -468,19 +488,57 @@ def check_define(node, kind):
         report("'%s %s' replaces a Ren'Py built-in name, which may cause problems.", kind, node.varname)
 
 
+def check_style_property_displayable(name, property, d):
+
+    if not d._duplicatable:
+        check_displayable(
+            "{}, property {}".format(name, property),
+            d)
+        return
+
+    renpy.style.init_inspect()
+
+    def sort_short(l):
+        l = list(l)
+        l.sort(key=lambda a: len(a))
+        return l
+
+    alts = sort_short(renpy.style.prefix_alts)
+
+    for p in sort_short(renpy.style.affects.get(property, [ ])):
+        for prefix in alts:
+            rest = p[len(prefix):]
+            if rest in renpy.style.all_properties:
+                args = d._args.copy(prefix=prefix)
+                dd = d._duplicate(args)
+                dd._unique()
+
+                check_displayable(
+                    "{}, property {}".format(name, prefix + property),
+                    dd)
+
+                break
+
+        # print property, p
+
+
 def check_style(name, s):
 
     for p in s.properties:
         for k, v in p.iteritems():
 
-            kname = name + ", property " + k
-
             # Treat font specially.
             if k.endswith("font"):
-                check_file(name, v)
+                if isinstance(v, renpy.text.font.FontGroup):
+                    for f in v.fonts:
+                        check_file(name, f)
+                else:
+                    check_file(name, v)
 
             if isinstance(v, renpy.display.core.Displayable):
-                check_displayable(kname, v)
+                check_style_property_displayable(name, k, v)
+#                check_displayable(kname, v)
+
 
 def check_label(node):
 
@@ -502,13 +560,14 @@ def check_label(node):
 
 
 def check_styles():
-    for full_name, s in renpy.style.styles.iteritems(): # @UndefinedVariable
+    for full_name, s in renpy.style.styles.iteritems():  # @UndefinedVariable
         name = "style." + full_name[0]
         for i in full_name[1:]:
             name += "[{!r}]".format(i)
 
         check_style("Style " + name, s)
 
+
 def humanize(n):
     s = str(n)
 
@@ -522,6 +581,7 @@ def humanize(n):
 
     return ''.join(rv)
 
+
 def check_filename_encodings():
     """
     Checks files to ensure that they are displayable in unicode.
@@ -537,6 +597,7 @@ def check_filename_encodings():
         report("%s contains non-ASCII characters in its filename.", filename)
         add("(ZIP file distributions can only reliably include ASCII filenames.)")
 
+
 class Count(object):
     """
     Stores information about the word count.
@@ -557,6 +618,7 @@ class Count(object):
         self.words += len(s.split())
         self.characters += len(s)
 
+
 def common(n):
     """
     Returns true if the node is in the common directory.
@@ -569,6 +631,7 @@ def common(n):
     else:
         return False
 
+
 def lint():
     """
     The master lint function, that's responsible for staging all of the
@@ -586,8 +649,8 @@ def lint():
 
     renpy.game.lint = True
 
-    print codecs.BOM_UTF8
-    print unicode(renpy.version + " lint report, generated at: " + time.ctime()).encode("utf-8")
+    print(codecs.BOM_UTF8)
+    print(unicode(renpy.version + " lint report, generated at: " + time.ctime()).encode("utf-8"))
 
     # This supports check_hide.
     global image_prefixes
@@ -646,7 +709,6 @@ def lint():
 
             counts[language].add(node.what)
 
-
         elif isinstance(node, renpy.ast.Menu):
             check_menu(node)
             menu_count += 1
@@ -684,8 +746,6 @@ def lint():
         elif isinstance(node, renpy.ast.Default):
             check_define(node, "default")
 
-
-
     report_node = None
 
     check_styles()
@@ -694,7 +754,6 @@ def lint():
     for f in renpy.config.lint_hooks:
         f()
 
-
     lines = [ ]
 
     def report_language(language):
@@ -720,11 +779,10 @@ characters per block. """.format(
 
         lines.append(s)
 
-
-    print
-    print
-    print "Statistics:"
-    print
+    print()
+    print()
+    print("Statistics:")
+    print()
 
     languages = list(counts)
     languages.sort()
@@ -736,17 +794,16 @@ characters per block. """.format(
 
     for l in lines:
         for ll in textwrap.wrap(l, 78):
-            print ll.encode("utf-8")
+            print(ll.encode("utf-8"))
 
-        print
+        print()
 
-    print
-    if renpy.config.developer:
-        print "Remember to set config.developer to False before releasing."
-        print
+    print()
+    if renpy.config.developer and (renpy.config.original_developer != "auto"):
+        print("Remember to set config.developer to False before releasing.")
+        print()
 
-    print "Lint is not a substitute for thorough testing. Remember to update Ren'Py"
-    print "before releasing. New releases fix bugs and improve compatibility."
+    print("Lint is not a substitute for thorough testing. Remember to update Ren'Py")
+    print("before releasing. New releases fix bugs and improve compatibility.")
 
     return False
-
