@@ -1,5 +1,4 @@
-
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,17 +22,21 @@
 # This module contains the parser for the Ren'Py script language. It's
 # called when parsing is necessary, and creates an AST from the script.
 
+from __future__ import print_function
 import codecs
 import re
 import os
 
 import renpy.display
+import renpy.test
+
 import renpy.ast as ast
 import renpy.sl2
 
 # A list of parse error messages.
 parse_errors = [ ]
 
+
 class ParseError(Exception):
 
     def __init__(self, filename, number, msg, line=None, pos=None, first=False):
@@ -84,6 +87,8 @@ class ParseError(Exception):
         return self.message
 
 # Something to hold the expected line number.
+
+
 class LineNumberHolder(object):
     """
     Holds the expected line number.
@@ -92,6 +97,7 @@ class LineNumberHolder(object):
     def __init__(self):
         self.line = 0
 
+
 def unicode_filename(fn):
     """
     Converts the supplied filename to unicode.
@@ -119,6 +125,7 @@ def unicode_filename(fn):
 # before this.
 lllword = re.compile(r'__(\w+)|\w+| +|.', re.S)
 
+
 def munge_filename(fn):
     # The prefix that's used when __ is found in the file.
     rv = os.path.basename(fn)
@@ -132,6 +139,7 @@ def munge_filename(fn):
 
     return "_m1_" + rv + "__"
 
+
 def elide_filename(fn):
     """
     Returns a version of fn that is either relative to the base directory,
@@ -149,6 +157,7 @@ def elide_filename(fn):
     else:
         return fn.replace("\\", "/")
 
+
 def unelide_filename(fn):
     fn1 = os.path.join(renpy.config.basedir, fn)
     if os.path.exists(fn1):
@@ -163,6 +172,7 @@ def unelide_filename(fn):
 # The filename that the start and end positions are relative to.
 original_filename = ""
 
+
 def list_logical_lines(filename, filedata=None, linenumber=1):
     """
     Reads `filename`, and divides it into logical lines.
@@ -239,16 +249,17 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
                     # Add to the results.
                     rv.append((filename, start_number, line))
 
-                    if endpos is None:
-                        endpos = pos
+                if endpos is None:
+                    endpos = pos
 
-                    lines[loc].end_delim = endpos + 1
+                lines[loc].end_delim = endpos + 1
 
-                    while data[endpos-1] in ' \r':
-                        endpos -= 1
+                while data[endpos-1] in ' \r':
+                    endpos -= 1
 
-                    lines[loc].end = endpos
-                    lines[loc].text = data[lines[loc].start:lines[loc].end]
+                lines[loc].end = endpos
+                lines[loc].text = data[lines[loc].start:lines[loc].end]
+                lines[loc].full_text = data[lines[loc].start:lines[loc].end_delim]
 
                 pos += 1
                 number += 1
@@ -343,14 +354,12 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
 
             pos = m.end(0)
 
-
     if not line == "":
         raise ParseError(filename, start_number, "is not terminated with a newline. (Check strings and parenthesis.)", line=line, first=True)
 
     return rv
 
 
-
 def group_logical_lines(lines):
     """
     This takes as input the list of logical line triples output from
@@ -418,69 +427,71 @@ def group_logical_lines(lines):
 # A list of keywords which should not be parsed as names, because
 # there is a huge chance of confusion.
 #
-# Note: We need to be careful with what's in here, because thse
+# Note: We need to be careful with what's in here, because these
 # are banned in simple_expressions, where we might want to use
 # some of them.
 KEYWORDS = set([
-            '$',
-            'as',
-            'at',
-            'behind',
-            'call',
-            'expression',
-            'hide',
-            'if',
-            'in',
-            'image',
-            'init',
-            'jump',
-            'menu',
-            'onlayer',
-            'python',
-            'return',
-            'scene',
-            'set',
-            'show',
-            'with',
-            'while',
-            'zorder',
-            'transform',
-            ])
+    '$',
+    'as',
+    'at',
+    'behind',
+    'call',
+    'expression',
+    'hide',
+    'if',
+    'in',
+    'image',
+    'init',
+    'jump',
+    'menu',
+    'onlayer',
+    'python',
+    'return',
+    'scene',
+    'set',
+    'show',
+    'with',
+    'while',
+    'zorder',
+    'transform',
+    ])
 
 OPERATORS = [
-        '<',
-        '<=',
-        '>',
-        '>=',
-        '<>',
-        '!=',
-        '==',
-        '|',
-        '^',
-        '&',
-        '<<',
-        '>>',
-        '+',
-        '-',
-        '*',
-        '/',
-        '//',
-        '%',
-        '~',
-        '**',
-        ]
+    '<',
+    '<=',
+    '>',
+    '>=',
+    '<>',
+    '!=',
+    '==',
+    '|',
+    '^',
+    '&',
+    '<<',
+    '>>',
+    '+',
+    '-',
+    '*',
+    '/',
+    '//',
+    '%',
+    '~',
+    '**',
+    ]
 
 ESCAPED_OPERATORS = [
-        r'\bor\b',
-        r'\band\b',
-        r'\bnot\b',
-        r'\bin\b',
-        r'\bis\b',
+    r'\bor\b',
+    r'\band\b',
+    r'\bnot\b',
+    r'\bin\b',
+    r'\bis\b',
     ]
 
 operator_regexp = "|".join([ re.escape(i) for i in OPERATORS ] + ESCAPED_OPERATORS)
 
 word_regexp = ur'[a-zA-Z_\u00a0-\ufffd][0-9a-zA-Z_\u00a0-\ufffd]*'
+image_word_regexp = ur'[-0-9a-zA-Z_\u00a0-\ufffd][-0-9a-zA-Z_\u00a0-\ufffd]*'
+
 
 class Lexer(object):
     """
@@ -489,11 +500,14 @@ class Lexer(object):
     sub-lexers to lex sub-blocks.
     """
 
-    def __init__(self, block, init=False):
+    def __init__(self, block, init=False, init_offset=0, global_label=None):
 
         # Are we underneath an init block?
         self.init = init
 
+        # The priority of auto-defined init statements.
+        self.init_offset = init_offset
+
         self.block = block
         self.eob = False
 
@@ -504,12 +518,12 @@ class Lexer(object):
         self.text = ""
         self.number = 0
         self.subblock = [ ]
+        self.global_label = global_label
         self.pos = 0
         self.word_cache_pos = -1
         self.word_cache_newpos = -1
         self.word_cache = ""
 
-
     def advance(self):
         """
         Advances this lexer to the next line in the block. The lexer
@@ -576,7 +590,6 @@ class Lexer(object):
         self.skip_whitespace()
         return self.match_regexp(regexp)
 
-
     def keyword(self, word):
         """
         Matches a keyword at the current position. A keyword is a word
@@ -591,7 +604,6 @@ class Lexer(object):
         self.pos = oldpos
         return ''
 
-
     def error(self, msg):
         """
         Convenience function for reporting a parse error at the current
@@ -638,7 +650,6 @@ class Lexer(object):
         if not self.subblock:
             self.error('%s expects a non-empty block.' % stmt)
 
-
     def subblock_lexer(self, init=False):
         """
         Returns a new lexer object, equiped to parse the block
@@ -647,7 +658,7 @@ class Lexer(object):
 
         init = self.init or init
 
-        return Lexer(self.subblock, init=init)
+        return Lexer(self.subblock, init=init, init_offset=self.init_offset, global_label=self.global_label)
 
     def string(self):
         """
@@ -702,7 +713,7 @@ class Lexer(object):
 
         return self.match(r'(\+|\-)?\d+')
 
-    def float(self): #@ReservedAssignment
+    def float(self):  # @ReservedAssignment
         """
         Tries to parse a number (float). Returns a string containing the
         number, or None.
@@ -712,7 +723,7 @@ class Lexer(object):
 
     def hash(self):
         """
-        Matches the chatacters in an md5 hash, and then some.
+        Matches the characters in an md5 hash, and then some.
         """
 
         return self.match(r'\w+')
@@ -733,7 +744,6 @@ class Lexer(object):
 
         return rv
 
-
     def name(self):
         """
         This tries to parse a name. Returns the name or None.
@@ -748,6 +758,76 @@ class Lexer(object):
 
         return rv
 
+    def set_global_label(self, label):
+        """
+        Set current global_label, which is used for label_name calculations.
+        label can be any valid label or None, but this has only effect if label
+        has global part.
+        """
+        if label and label[0] != '.':
+            self.global_label = label.split('.')[0]
+
+    def label_name(self, declare=False):
+        """
+        Try to parse label name. Returns name in form of "global.local" if local
+        is present, "global" otherwise; or None if it doesn't parse.
+
+        If declare is True, allow only such names that are valid for declaration
+        (e.g. forbid global name mismatch)
+        """
+
+        old_pos = self.pos
+        local_name = None
+        global_name = self.name()
+
+        if not global_name:
+            # .local label
+            if not self.match('\.') or not self.global_label:
+                self.pos = old_pos
+                return None
+            global_name = self.global_label
+            local_name = self.name()
+            if not local_name:
+                self.pos = old_pos
+                return None
+        else:
+            if self.match('\.'):
+                # full global.local name
+                if declare and global_name != self.global_label:
+                    self.pos = old_pos
+                    return None
+
+                local_name = self.name()
+                if not local_name:
+                    self.pos = old_pos
+                    return None
+
+        if not local_name:
+            return global_name
+
+        return global_name+'.'+local_name
+
+    def label_name_declare(self):
+        """
+        Same as label_name, but set declare to True.
+        """
+        return self.label_name(declare=True)
+
+    def image_name_component(self):
+        """
+        Matches a word that is a component of an image name. (These are
+        strings of numbers, letters, and underscores.)
+        """
+
+        oldpos = self.pos
+        rv = self.match(image_word_regexp)
+
+        if rv in KEYWORDS:
+            self.pos = oldpos
+            return None
+
+        return rv
+
     def python_string(self):
         """
         This tries to match a python string at the current
@@ -797,7 +877,6 @@ class Lexer(object):
         self.pos += 1
         return True
 
-
     def dotted_name(self):
         """
         This tries to match a dotted name, which is one or more names,
@@ -840,7 +919,7 @@ class Lexer(object):
             if c in delim:
                 return renpy.ast.PyExpr(self.text[start:self.pos], self.filename, self.number)
 
-            if c == '"' or c == "'":
+            if c in "'\"":
                 self.python_string()
                 continue
 
@@ -862,7 +941,7 @@ class Lexer(object):
         if not pe:
             self.error("expected python_expression")
 
-        rv = renpy.ast.PyExpr(pe.strip(), pe.filename, pe.linenumber) # E1101
+        rv = renpy.ast.PyExpr(pe.strip(), pe.filename, pe.linenumber)  # E1101
 
         return rv
 
@@ -870,7 +949,7 @@ class Lexer(object):
         """
         Tries to match a parenthesised python expression. If it can,
         returns true and updates the current position to be after the
-        closing parenthesis. Returns False otherewise.
+        closing parenthesis. Returns False otherwise.
         """
 
         c = self.text[self.pos]
@@ -887,7 +966,6 @@ class Lexer(object):
             self.pos += 1
             return True
 
-
         if c == '{':
             self.pos += 1
             self.delimited_python('}')
@@ -896,7 +974,6 @@ class Lexer(object):
 
         return False
 
-
     def simple_expression(self, comma=False):
         """
         Tries to parse a simple_expression. Returns the text if it can, or
@@ -917,9 +994,9 @@ class Lexer(object):
             # We start with either a name, a python_string, or parenthesized
             # python
             if not (self.python_string() or
-                self.name() or
-                self.float() or
-                self.parenthesised_python()):
+                    self.name() or
+                    self.float() or
+                    self.parenthesised_python()):
 
                 break
 
@@ -966,7 +1043,6 @@ class Lexer(object):
 
         return self.simple_expression(comma=True)
 
-
     def checkpoint(self):
         """
         Returns an opaque representation of the lexer state. This can be
@@ -1056,23 +1132,48 @@ class Lexer(object):
         process(self.subblock, '')
         return ''.join(rv)
 
-def parse_image_name(l):
+
+def parse_image_name(l, string=False, nodash=False):
     """
     This parses an image name, and returns it as a tuple. It requires
     that the image name be present.
     """
 
-    rv = [ l.require(l.name) ]
+    points = [ l.checkpoint() ]
+    rv = [ l.require(l.image_name_component) ]
 
     while True:
-        n = l.simple_expression()
+
+        points.append(l.checkpoint())
+
+        n = l.image_name_component()
+
         if not n:
+            points.pop()
             break
 
         rv.append(n.strip())
 
+    if string:
+        points.append(l.checkpoint())
+
+        s = l.simple_expression()
+
+        if s is not None:
+            rv.append(unicode(s))
+        else:
+            points.pop()
+
+    if nodash:
+        for i, p in zip(rv, points):
+            if i and i[0] == '-':
+                l.revert(p)
+                l.skip_whitespace()
+                l.error("image name components may not begin with a '-'.")
+
     return tuple(rv)
 
+
 def parse_simple_expression_list(l):
     """
     This parses a comma-separated list of simple_expressions, and
@@ -1095,6 +1196,7 @@ def parse_simple_expression_list(l):
 
     return rv
 
+
 def parse_image_specifier(l):
     """
     This parses an image specifier.
@@ -1110,7 +1212,7 @@ def parse_image_specifier(l):
         expression = l.require(l.simple_expression)
         image_name = ( expression.strip(), )
     else:
-        image_name = parse_image_name(l)
+        image_name = parse_image_name(l, True)
         expression = None
 
     while True:
@@ -1165,13 +1267,9 @@ def parse_image_specifier(l):
 
         break
 
-    if layer is None:
-        layer = 'master'
-
-
-
     return image_name, expression, tag, at_list, layer, zorder, behind
 
+
 def parse_with(l, node):
     """
     Tries to parse the with clause associated with this statement. If
@@ -1202,7 +1300,7 @@ def parse_menu(stmtl, loc):
     has_caption = False
 
     with_ = None
-    set = None #@ReservedAssignment
+    set = None  # @ReservedAssignment
 
     say_who = None
     say_what = None
@@ -1223,7 +1321,7 @@ def parse_menu(stmtl, loc):
             continue
 
         if l.keyword('set'):
-            set = l.require(l.simple_expression) #@ReservedAssignment
+            set = l.require(l.simple_expression)  # @ReservedAssignment
             l.expect_eol()
             l.expect_noblock('set menuitem')
             l.advance()
@@ -1257,7 +1355,6 @@ def parse_menu(stmtl, loc):
 
         l.revert(state)
 
-
         label = l.string()
 
         if label is None:
@@ -1341,7 +1438,6 @@ def parse_parameters(l):
 
             names.add(extrakw)
 
-
         elif l.match(r'\*'):
 
             if not add_positional:
@@ -1410,7 +1506,6 @@ def parse_arguments(l):
 
             extrakw = l.delimited_python("),")
 
-
         elif l.match(r'\*'):
             if extrapos is not None:
                 l.error('a call may have only one * argument')
@@ -1468,7 +1563,7 @@ class ParseTrie(object):
 
         word = l.word() or l.match(r'\$')
 
-        if not word in self.words:
+        if word not in self.words:
             l.pos = old_pos
             return self.default
 
@@ -1564,7 +1659,8 @@ def pass_statement(l, loc):
 @statement("menu")
 def menu_statement(l, loc):
     l.expect_block('menu statement')
-    label = l.name()
+    label = l.label_name_declare()
+    l.set_global_label(label)
     l.require(':')
     l.expect_eol()
 
@@ -1605,7 +1701,7 @@ def jump_statement(l, loc):
         target = l.require(l.simple_expression)
     else:
         expression = False
-        target = l.require(l.name)
+        target = l.require(l.label_name)
 
     l.expect_eol()
     l.advance()
@@ -1623,7 +1719,7 @@ def call_statement(l, loc):
 
     else:
         expression = False
-        target = l.require(l.name)
+        target = l.require(l.label_name)
 
     # Optional pass, to let someone write:
     # call expression foo pass (bar, baz)
@@ -1634,13 +1730,15 @@ def call_statement(l, loc):
     rv = [ ast.Call(loc, target, expression, arguments) ]
 
     if l.keyword('from'):
-        name = l.require(l.name)
+        name = l.require(l.label_name_declare)
+        l.set_global_label(name)
         rv.append(ast.Label(loc, name, [], None))
     else:
-        if expression:
-            renpy.add_from.report_missing("expression", original_filename, renpy.scriptedit.lines[loc].end)
-        else:
-            renpy.add_from.report_missing(target, original_filename, renpy.scriptedit.lines[loc].end)
+        if renpy.scriptedit.lines and (loc in renpy.scriptedit.lines):
+            if expression:
+                renpy.add_from.report_missing("expression", original_filename, renpy.scriptedit.lines[loc].end)
+            else:
+                renpy.add_from.report_missing(target, original_filename, renpy.scriptedit.lines[loc].end)
 
     rv.append(ast.Pass(loc))
 
@@ -1693,6 +1791,7 @@ def show_statement(l, loc):
 
     return rv
 
+
 @statement("show layer")
 def show_layer_statement(l, loc):
 
@@ -1741,7 +1840,7 @@ def with_statement(l, loc):
 
 @statement("image")
 def image_statement(l, loc):
-    name = parse_image_name(l)
+    name = parse_image_name(l, nodash=True)
 
     if l.match(':'):
         l.expect_eol()
@@ -1761,7 +1860,7 @@ def image_statement(l, loc):
     rv = ast.Image(loc, name, expr, atl)
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], 990)
+        rv = ast.Init(loc, [ rv ], 500 + l.init_offset)
 
     l.advance()
 
@@ -1778,11 +1877,11 @@ def define_statement(l, loc):
         priority = 0
 
     store = 'store'
-    name = l.require(l.name)
+    name = l.require(l.word)
 
     while l.match(r'\.'):
         store = store + "." + name
-        name = l.require(l.name)
+        name = l.require(l.word)
 
     l.require('=')
     expr = l.rest()
@@ -1795,7 +1894,7 @@ def define_statement(l, loc):
     rv = ast.Define(loc, store, name, expr)
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], priority)
+        rv = ast.Init(loc, [ rv ], priority + l.init_offset)
 
     l.advance()
 
@@ -1812,11 +1911,11 @@ def default_statement(l, loc):
         priority = 0
 
     store = 'store'
-    name = l.require(l.name)
+    name = l.require(l.word)
 
     while l.match(r'\.'):
         store = store + "." + name
-        name = l.require(l.name)
+        name = l.require(l.word)
 
     l.require('=')
     expr = l.rest()
@@ -1829,7 +1928,7 @@ def default_statement(l, loc):
     rv = ast.Default(loc, store, name, expr)
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], priority)
+        rv = ast.Init(loc, [ rv ], priority + l.init_offset)
 
     l.advance()
 
@@ -1859,7 +1958,7 @@ def transform_statement(l, loc):
     rv = ast.Transform(loc, name, atl, parameters)
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], priority)
+        rv = ast.Init(loc, [ rv ], priority + l.init_offset)
 
     l.advance()
 
@@ -1892,7 +1991,7 @@ def python_statement(l, loc):
         hide = True
 
     if l.keyword('in'):
-        store = "store." + l.require(l.name)
+        store = "store." + l.require(l.dotted_name)
 
     l.require(':')
     l.expect_block('python block')
@@ -1909,8 +2008,9 @@ def python_statement(l, loc):
 
 @statement("label")
 def label_statement(l, loc, init=False):
-    name = l.require(l.name)
 
+    name = l.require(l.label_name_declare)
+    l.set_global_label(name)
     parameters = parse_parameters(l)
 
     if l.keyword('hide'):
@@ -1928,10 +2028,26 @@ def label_statement(l, loc, init=False):
     l.advance()
     return ast.Label(loc, name, block, parameters, hide=hide)
 
+
+@statement("init offset")
+def init_offset_statement(l, loc):
+
+    l.require('=')
+    offset = l.require(l.integer)
+
+    l.expect_eol()
+    l.expect_noblock('init offset statement')
+    l.advance()
+
+    l.init_offset = int(offset)
+    return [ ]
+
+
 @statement("init label")
 def init_label_statement(l, loc):
     return label_statement(l, loc, init=True)
 
+
 @statement("init")
 def init_statement(l, loc):
 
@@ -1962,7 +2078,7 @@ def init_statement(l, loc):
         finally:
             l.init = old_init
 
-    return ast.Init(loc, block, priority)
+    return ast.Init(loc, block, priority + l.init_offset)
 
 
 def screen1_statement(l, loc):
@@ -1979,7 +2095,7 @@ def screen1_statement(l, loc):
     rv = ast.Screen(loc, screen)
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], -500)
+        rv = ast.Init(loc, [ rv ], -500 + l.init_offset)
 
     return rv
 
@@ -1995,13 +2111,14 @@ def screen2_statement(l, loc):
     rv = ast.Screen(loc, screen)
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], -500)
+        rv = ast.Init(loc, [ rv ], -500 + l.init_offset)
 
     return rv
 
 # The version of screen language to use by default.
 default_screen_language = int(os.environ.get("RENPY_SCREEN_LANGUAGE", "2"))
 
+
 @statement("screen")
 def screen_statement(l, loc):
 
@@ -2019,6 +2136,25 @@ def screen_statement(l, loc):
         l.error("Bad screen language version.")
 
 
+@statement("testcase")
+def testcase_statement(l, loc):
+    name = l.require(l.name)
+    l.require(':')
+    l.expect_eol()
+    l.expect_block('testcase statement')
+
+    test = renpy.test.testparser.parse_block(l.subblock_lexer(), loc)
+
+    l.advance()
+
+    rv = ast.Testcase(loc, name, test)
+
+    if not l.init:
+        rv = ast.Init(loc, [ rv ], 500 + l.init_offset)
+
+    return rv
+
+
 def translate_strings(init_loc, language, l):
     l.require(':')
     l.expect_eol()
@@ -2055,13 +2191,15 @@ def translate_strings(init_loc, language, l):
             if old is None:
                 ll.error('no string to translate')
 
+            newloc = ll.get_location()
             new = parse_string(ll.rest())
 
-            block.append(renpy.ast.TranslateString(loc, language, old, new))
+            block.append(renpy.ast.TranslateString(loc, language, old, new, newloc))
 
             old = None
             new = None
             loc = None
+            newloc = None
 
         else:
             ll.error('unknown statement')
@@ -2074,7 +2212,8 @@ def translate_strings(init_loc, language, l):
     if l.init:
         return block
 
-    return ast.Init(init_loc, block, 0)
+    return ast.Init(init_loc, block, l.init_offset)
+
 
 @statement("translate")
 def translate_statement(l, loc):
@@ -2095,7 +2234,7 @@ def translate_statement(l, loc):
             l.init = True
 
             block = [ python_statement(l, loc) ]
-            return [ ast.TranslateBlock(loc, language, block) ]
+            return [ ast.TranslateEarlyBlock(loc, language, block) ]
         finally:
             l.init = old_init
 
@@ -2109,7 +2248,6 @@ def translate_statement(l, loc):
         finally:
             l.init = old_init
 
-
     l.require(':')
     l.expect_eol()
 
@@ -2156,7 +2294,7 @@ def style_statement(l, loc):
         if l.keyword("del"):
             propname = l.require(l.name)
 
-            if propname not in renpy.style.prefixed_all_properties: # @UndefinedVariable
+            if propname not in renpy.style.prefixed_all_properties:  # @UndefinedVariable
                 l.error("style property %s is not known." % propname)
 
             rv.delattr.append(propname)
@@ -2173,7 +2311,7 @@ def style_statement(l, loc):
         propname = l.name()
 
         if propname is not None:
-            if propname not in renpy.style.prefixed_all_properties: # @UndefinedVariable
+            if (propname != "properties") and (propname not in renpy.style.prefixed_all_properties):  # @UndefinedVariable
                 l.error("style property %s is not known." % propname)
 
             if propname in rv.properties:
@@ -2205,7 +2343,7 @@ def style_statement(l, loc):
             ll.expect_eol()
 
     if not l.init:
-        rv = ast.Init(loc, [ rv ], 0)
+        rv = ast.Init(loc, [ rv ], l.init_offset)
 
     l.advance()
 
@@ -2242,7 +2380,7 @@ def say_statement(l, loc):
         if not prefix:
             prefix = ""
 
-        component = l.word()
+        component = l.image_name_component()
 
         if component is None:
             break
@@ -2299,7 +2437,6 @@ def parse_statement(l):
     return pf(l, loc)
 
 
-
 def parse_block(l):
     """
     This parses a block of Ren'Py statements. It returns a list of the
@@ -2320,12 +2457,13 @@ def parse_block(l):
             else:
                 rv.append(stmt)
 
-        except ParseError, e:
+        except ParseError as e:
             parse_errors.append(e.message)
             l.advance()
 
     return rv
 
+
 def parse(fn, filedata=None, linenumber=1):
     """
     Parses a Ren'Py script contained within the file `fn`.
@@ -2360,12 +2498,14 @@ def parse(fn, filedata=None, linenumber=1):
 
     return rv
 
+
 def get_parse_errors():
     global parse_errors
     rv = parse_errors
     parse_errors = [ ]
     return rv
 
+
 def report_parse_errors():
 
     if not parse_errors:
@@ -2376,9 +2516,9 @@ def report_parse_errors():
     f, error_fn = renpy.error.open_error_file("errors.txt", "w")
     f.write(codecs.BOM_UTF8)
 
-    print >>f, "I'm sorry, but errors were detected in your script. Please correct the"
-    print >>f, "errors listed below, and try again."
-    print >>f
+    print("I'm sorry, but errors were detected in your script. Please correct the", file=f)
+    print("errors listed below, and try again.", file=f)
+    print(file=f)
 
     for i in parse_errors:
 
@@ -2390,21 +2530,20 @@ def report_parse_errors():
         except:
             pass
 
-        print
-        print >>f
-        print i
-        print >>f, i
+        print()
+        print(file=f)
+        print(i)
+        print(i, file=f)
 
-
-    print >>f
-    print >>f, "Ren'Py Version:", renpy.version
+    print(file=f)
+    print("Ren'Py Version:", renpy.version, file=f)
 
     f.close()
 
     renpy.display.error.report_parse_errors(full_text, error_fn)
 
     try:
-        if renpy.game.args.command == "run": #@UndefinedVariable
+        if renpy.game.args.command == "run":  # @UndefinedVariable
             renpy.exports.launch_editor([ error_fn ], 1, transient=1)
     except:
         pass
